@@ -49,8 +49,8 @@ void HLCmdLineApp::printVersionInfo()
          << "\n Copyright (C) 2002-2013 Andre Simon <andre.simon1 at gmx.de>"
          << "\n\n Argparser class"
          << "\n Copyright (C) 2006-2008 Antonio Diaz Diaz <ant_diaz at teleline.es>"
-         << "\n\n Artistic Style Classes (2.02.1)"
-         << "\n Copyright (C) 2006-2011 by Jim Pattee <jimp03 at email.com>"
+         << "\n\n Artistic Style Classes (2.03)"
+         << "\n Copyright (C) 2006-2013 by Jim Pattee <jimp03 at email.com>"
          << "\n Copyright (C) 1998-2002 by Tal Davidson"
          << "\n\n Diluculum Lua wrapper (1.0)"
          << "\n Copyright (C) 2005-2013 by Leandro Motta Barros"
@@ -209,6 +209,9 @@ void HLCmdLineApp::printConfigInfo ( )
 #endif
 #ifdef HL_CONFIG_DIR
     cout << "Compiler directive HL_CONFIG_DIR = " <<HL_CONFIG_DIR<< "\n";
+#endif
+#ifdef USE_LUA52
+    cout << "\nLua 5.2 compilation enabled\n";
 #endif
     cout << endl;
 }
