@@ -1,7 +1,7 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *   ASResource.cpp
  *
- *   Copyright (C) 2006-2011 by Jim Pattee <jimp03@email.com>
+ *   Copyright (C) 2006-2013 by Jim Pattee <jimp03@email.com>
  *   Copyright (C) 1998-2002 by Tal Davidson
  *   <http://www.gnu.org/licenses/lgpl-3.0.html>
  *
@@ -25,7 +25,7 @@
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  */
 
-#include "astyle/astyle.h"
+#include "../../include/astyle/astyle.h"
 #include <algorithm>
 
 
@@ -56,6 +56,7 @@ const string ASResource::AS_OPERATOR = string("operator");
 const string ASResource::AS_TEMPLATE = string("template");
 const string ASResource::AS_TRY = string("try");
 const string ASResource::AS_CATCH = string("catch");
+const string ASResource::AS_THROW = string("throw");
 const string ASResource::AS_FINALLY = string("finally");
 const string ASResource::_AS_TRY = string("__try");
 const string ASResource::_AS_FINALLY = string("__finally");
@@ -95,8 +96,8 @@ const string ASResource::AS_AND_ASSIGN = string("&=");
 const string ASResource::AS_XOR_ASSIGN = string("^=");
 const string ASResource::AS_GR_GR_ASSIGN = string(">>=");
 const string ASResource::AS_LS_LS_ASSIGN = string("<<=");
-const string ASResource::AS_GR_GR_GR_ASSIGN = string(">>>=");	// Java only?
-const string ASResource::AS_LS_LS_LS_ASSIGN = string("<<<=");	// Java only?
+const string ASResource::AS_GR_GR_GR_ASSIGN = string(">>>=");
+const string ASResource::AS_LS_LS_LS_ASSIGN = string("<<<=");
 const string ASResource::AS_GCC_MIN_ASSIGN = string("<?");
 const string ASResource::AS_GCC_MAX_ASSIGN = string(">?");
 
@@ -111,12 +112,12 @@ const string ASResource::AS_MINUS_MINUS = string("--");
 const string ASResource::AS_NOT_EQUAL = string("!=");
 const string ASResource::AS_GR_EQUAL = string(">=");
 const string ASResource::AS_GR_GR = string(">>");
-const string ASResource::AS_GR_GR_GR = string(">>>");			// Java only?
+const string ASResource::AS_GR_GR_GR = string(">>>");
 const string ASResource::AS_LS_EQUAL = string("<=");
 const string ASResource::AS_LS_LS = string("<<");
-const string ASResource::AS_LS_LS_LS = string("<<<");			// Java only?
+const string ASResource::AS_LS_LS_LS = string("<<<");
 const string ASResource::AS_QUESTION_QUESTION = string("??");
-const string ASResource::AS_EQUAL_GR = string("=>");            // C# lambda expression arrow
+const string ASResource::AS_LAMBDA = string("=>");            // C# lambda expression arrow
 const string ASResource::AS_ARROW = string("->");
 const string ASResource::AS_AND = string("&&");
 const string ASResource::AS_OR = string("||");
@@ -314,7 +315,7 @@ void ASResource::buildNonAssignmentOperators(vector<const string*>* nonAssignmen
 	nonAssignmentOperators->push_back(&AS_ARROW);
 	nonAssignmentOperators->push_back(&AS_AND);
 	nonAssignmentOperators->push_back(&AS_OR);
-	nonAssignmentOperators->push_back(&AS_EQUAL_GR);
+	nonAssignmentOperators->push_back(&AS_LAMBDA);
 
 	sort(nonAssignmentOperators->begin(), nonAssignmentOperators->end(), sortOnLength);
 }
@@ -374,7 +375,7 @@ void ASResource::buildNonParenHeaders(vector<const string*>* nonParenHeaders, in
  *
  * @param operators             a reference to the vector to be built.
  */
-void ASResource::buildOperators(vector<const string*>* operators)
+void ASResource::buildOperators(vector<const string*>* operators, int fileType)
 {
 	operators->push_back(&AS_PLUS_ASSIGN);
 	operators->push_back(&AS_MINUS_ASSIGN);
@@ -399,9 +400,7 @@ void ASResource::buildOperators(vector<const string*>* operators)
 	operators->push_back(&AS_LS_LS_LS);
 	operators->push_back(&AS_LS_LS);
 	operators->push_back(&AS_QUESTION_QUESTION);
-	operators->push_back(&AS_EQUAL_GR);
-	operators->push_back(&AS_GCC_MIN_ASSIGN);
-	operators->push_back(&AS_GCC_MAX_ASSIGN);
+	operators->push_back(&AS_LAMBDA);
 	operators->push_back(&AS_ARROW);
 	operators->push_back(&AS_AND);
 	operators->push_back(&AS_OR);
@@ -421,7 +420,11 @@ void ASResource::buildOperators(vector<const string*>* operators)
 	operators->push_back(&AS_BIT_AND);
 	operators->push_back(&AS_BIT_NOT);
 	operators->push_back(&AS_BIT_XOR);
-
+	if (fileType == C_TYPE)
+	{
+		operators->push_back(&AS_GCC_MIN_ASSIGN);
+		operators->push_back(&AS_GCC_MAX_ASSIGN);
+	}
 	sort(operators->begin(), operators->end(), sortOnLength);
 }
 
@@ -523,7 +526,7 @@ void ASResource::buildPreDefinitionHeaders(vector<const string*>* preDefinitionH
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 // check if a specific line position contains a keyword.
-bool ASBase::findKeyword(const string& line, int i, const string& keyword) const
+bool ASBase::findKeyword(const string &line, int i, const string &keyword) const
 {
 	assert(isCharPotentialHeader(line, i));
 	// check the word
@@ -547,7 +550,7 @@ bool ASBase::findKeyword(const string& line, int i, const string& keyword) const
 
 // get the current word on a line
 // index must point to the beginning of the word
-string ASBase::getCurrentWord(const string& line, size_t index) const
+string ASBase::getCurrentWord(const string &line, size_t index) const
 {
 	assert(isCharPotentialHeader(line, index));
 	size_t lineLength = line.length();
