@@ -503,7 +503,7 @@ void MainWindow::on_action_About_Highlight_triggered()
                          "Highlight GUI %1\n"
                          "(C) 2002-2013 Andre Simon <andre.simon1 at gmx.de>\n\n"
                          "Artistic Style Classes\n(C) 1998-2002 Tal Davidson\n"
-                         "(C) 2006-2011 Jim Pattee <jimp03 at email.com>\n\n"
+                         "(C) 2006-2013 Jim Pattee <jimp03 at email.com>\n\n"
                          "Diluculum Lua wrapper\n"
 			 "(C) 2005-2013 by Leandro Motta Barros\n\n"
                          "Built with Qt version %2\n\n"
@@ -968,7 +968,8 @@ void MainWindow::highlight2Clipboard(bool getDataFromCP){
          ui->stackedSpecificOptions->setCurrentIndex(5);
          break;
   }
-     ui->tabWidget->setTabText(1, tr("%1 options").arg(ui->comboFormat->currentText() ));
+     ui->tabWidgetOptions->setTabText(1, tr("%1 options").arg(ui->comboFormat->currentText() ));
+     //ui->tabWidget->setTabText(1, tr("%1 options").arg(ui->comboFormat->currentText() ));
  }
 
  void MainWindow::updatePreview() {
