@@ -1,7 +1,7 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *   astyle.h
  *
- *   Copyright (C) 2006-2011 by Jim Pattee <jimp03@email.com>
+ *   Copyright (C) 2006-2013 by Jim Pattee <jimp03@email.com>
  *   Copyright (C) 1998-2002 by Tal Davidson
  *   <http://www.gnu.org/licenses/lgpl-3.0.html>
  *
@@ -35,18 +35,32 @@
 #include <cassert>
 #endif
 
-#include <string.h>         // need both string and string.h for GCC
+#include <cctype>
+#include <iostream>		// for cout
 #include <string>
 #include <vector>
-#include <cctype>
 
+#if defined(__GNUC__)
+#include <string.h>		// need both string and string.h for GCC
+#endif
+
+// define STDCALL and EXPORT for Windows
+// MINGW defines STDCALL in Windows.h (actually windef.h)
+// EXPORT has no value for Visual C if ASTYLE_NO_VCX (no VC Exports) is defined
 #ifdef _WIN32
+#ifndef STDCALL
 #define STDCALL __stdcall
-#define EXPORT  __declspec(dllexport)
+#endif
+#if defined(_MSC_VER) && defined(ASTYLE_NO_VCX)
+#define EXPORT
+#else
+#define EXPORT __declspec(dllexport)
+#endif
+// define STDCALL and EXPORT for non-Windows
 #else
 #define STDCALL
 #define EXPORT
-#endif
+#endif	// #ifdef _WIN32
 
 #ifdef _MSC_VER
 #pragma warning(disable: 4996)  // secure version deprecation warnings
@@ -181,7 +195,7 @@ class ASResource
 		void buildIndentableHeaders(vector<const string*>* indentableHeaders);
 		void buildNonAssignmentOperators(vector<const string*>* nonAssignmentOperators);
 		void buildNonParenHeaders(vector<const string*>* nonParenHeaders, int fileType, bool beautifier=false);
-		void buildOperators(vector<const string*>* operators);
+		void buildOperators(vector<const string*>* operators, int fileType);
 		void buildPreBlockStatements(vector<const string*>* preBlockStatements, int fileType);
 		void buildPreCommandHeaders(vector<const string*>* preCommandHeaders, int fileType);
 		void buildPreDefinitionHeaders(vector<const string*>* preDefinitionHeaders, int fileType);
@@ -191,7 +205,8 @@ class ASResource
 		static const string AS_DO, AS_WHILE;
 		static const string AS_FOR;
 		static const string AS_SWITCH, AS_CASE, AS_DEFAULT;
-		static const string AS_TRY, AS_CATCH, AS_THROWS, AS_FINALLY, _AS_TRY, _AS_FINALLY, _AS_EXCEPT;
+		static const string AS_TRY, AS_CATCH, AS_THROW, AS_THROWS, AS_FINALLY;
+		static const string _AS_TRY, _AS_FINALLY, _AS_EXCEPT;
 		static const string AS_PUBLIC, AS_PROTECTED, AS_PRIVATE;
 		static const string AS_CLASS, AS_STRUCT, AS_UNION, AS_INTERFACE, AS_NAMESPACE;
 		static const string AS_EXTERN, AS_ENUM;
@@ -209,7 +224,7 @@ class ASResource
 		static const string AS_GCC_MIN_ASSIGN, AS_GCC_MAX_ASSIGN;
 		static const string AS_EQUAL, AS_PLUS_PLUS, AS_MINUS_MINUS, AS_NOT_EQUAL, AS_GR_EQUAL, AS_GR_GR_GR, AS_GR_GR;
 		static const string AS_LS_EQUAL, AS_LS_LS_LS, AS_LS_LS;
-		static const string AS_QUESTION_QUESTION, AS_EQUAL_GR;
+		static const string AS_QUESTION_QUESTION, AS_LAMBDA;
 		static const string AS_ARROW, AS_AND, AS_OR;
 		static const string AS_COLON_COLON;
 		static const string AS_PLUS, AS_MINUS, AS_MULT, AS_DIV, AS_MOD, AS_GR, AS_LS;
@@ -237,11 +252,10 @@ class ASBase
 		virtual ~ASBase() {}
 
 		// functions definitions are at the end of ASResource.cpp
-		bool findKeyword(const string& line, int i, const string& keyword) const;
-		string getCurrentWord(const string& line, size_t index) const;
+		bool findKeyword(const string &line, int i, const string &keyword) const;
+		string getCurrentWord(const string &line, size_t index) const;
 
 	protected:
-
 		void init(int fileTypeArg) { baseFileType = fileTypeArg; }
 		bool isCStyle() const { return (baseFileType == C_TYPE); }
 		bool isJavaStyle() const { return (baseFileType == JAVA_TYPE); }
@@ -257,14 +271,14 @@ class ASBase
 		bool isLegalNameChar(char ch) const {
 			if (isWhiteSpace(ch)) return false;
 			if ((unsigned) ch > 127) return false;
-			return (isalnum(ch)
+			return (isalnum((unsigned char)ch)
 			        || ch == '.' || ch == '_'
 			        || (isJavaStyle() && ch == '$')
 			        || (isSharpStyle() && ch == '@'));  // may be used as a prefix
 		}
 
 		// check if a specific character can be part of a header
-		bool isCharPotentialHeader(const string& line, size_t i) const {
+		bool isCharPotentialHeader(const string &line, size_t i) const {
 			assert(!isWhiteSpace(line[i]));
 			char prevCh = ' ';
 			if (i > 0) prevCh = line[i-1];
@@ -277,7 +291,7 @@ class ASBase
 		bool isCharPotentialOperator(char ch) const {
 			assert(!isWhiteSpace(ch));
 			if ((unsigned) ch > 127) return false;
-			return (ispunct(ch)
+			return (ispunct((unsigned char)ch)
 			        && ch != '{' && ch != '}'
 			        && ch != '(' && ch != ')'
 			        && ch != '[' && ch != ']'
@@ -290,7 +304,7 @@ class ASBase
 		bool isWhiteSpace(char ch) const { return (ch == ' ' || ch == '\t'); }
 
 		// peek at the next unread character.
-		char peekNextChar(const string& line, int i) const {
+		char peekNextChar(const string &line, int i) const {
 			char ch = ' ';
 			size_t peekNum = line.find_first_not_of(" \t", i + 1);
 			if (peekNum == string::npos)
@@ -311,9 +325,11 @@ class ASBeautifier : protected ASResource, protected ASBase
 		virtual ~ASBeautifier();
 		virtual void init(ASSourceIterator* iter);
 		void init();
-		virtual string beautify(const string& line);
-		void setTabIndentation(int length = 4, bool forceTabs = false);
+		virtual string beautify(const string &line);
+		void setDefaultTabLength();
+		void setForceTabXIndentation(int length);
 		void setSpaceIndentation(int length = 4);
+		void setTabIndentation(int length = 4, bool forceTabs = false);
 		void setMaxInStatementIndentLength(int max);
 		void setMinConditionalIndentOption(int min);
 		void setMinConditionalIndentLength();
@@ -329,37 +345,40 @@ class ASBeautifier : protected ASResource, protected ASBase
 		void setEmptyLineFill(bool state);
 		void setPreprocessorIndent(bool state);
 		int  getBeautifierFileType() const;
-		int  getFileType();
-		int  getIndentLength(void);
-		string getIndentString(void);
-		string getNextWord(const string& line, size_t currPos) const;
-		bool getBracketIndent(void);
-		bool getBlockIndent(void);
-		bool getCaseIndent(void);
-		bool getClassIndent(void);
-		bool getEmptyLineFill(void);
-		bool getForceTabIndentation(void);
-		bool getModeManuallySet(void);
-		bool getPreprocessorIndent(void);
-		bool getSwitchIndent(void);
+		int  getFileType() const;
+		int  getIndentLength(void) const;
+		int  getTabLength(void) const;
+		string getIndentString(void) const;
+		string getNextWord(const string &line, size_t currPos) const;
+		bool getBracketIndent(void) const;
+		bool getBlockIndent(void) const;
+		bool getCaseIndent(void) const;
+		bool getClassIndent(void) const;
+		bool getEmptyLineFill(void) const;
+		bool getForceTabIndentation(void) const;
+		bool getModeManuallySet(void) const;
+		bool getPreprocessorIndent(void) const;
+		bool getSwitchIndent(void) const;
 
 	protected:
 		void deleteBeautifierVectors();
-		const string* findHeader(const string& line, int i,
+		const string* findHeader(const string &line, int i,
 		                         const vector<const string*>* possibleHeaders) const;
-		const string* findOperator(const string& line, int i,
+		const string* findOperator(const string &line, int i,
 		                           const vector<const string*>* possibleOperators) const;
-		int getNextProgramCharDistance(const string& line, int i) const;
-		int  indexOf(vector<const string*> &container, const string* element);
+		int getNextProgramCharDistance(const string &line, int i) const;
+		int  indexOf(vector<const string*> &container, const string* element) const;
 		void setBlockIndent(bool state);
 		void setBracketIndent(bool state);
-		string trim(const string& str);
+		string trim(const string &str);
+		string rtrim(const string &str);
 
 		// variables set by ASFormatter - must be updated in activeBeautifierStack
 		int  inLineNumber;
 		int  horstmannIndentInStatement;
 		int  nonInStatementBracket;
 		bool lineCommentNoBeautify;
+		bool isElseHeaderIndent;
 		bool isNonInStatementArray;
 		bool isSharpAccessor;
 		bool isSharpDelegate;
@@ -367,31 +386,34 @@ class ASBeautifier : protected ASResource, protected ASBase
 		bool isInBeautifySQL;
 		bool isInIndentableStruct;
 
-	private:
-		ASBeautifier(const ASBeautifier& copy);
-		ASBeautifier& operator=(ASBeautifier&);        // not to be implemented
+	private:  // functions
+		ASBeautifier(const ASBeautifier &copy);
+		ASBeautifier &operator=(ASBeautifier &);       // not to be implemented
 
 		void computePreliminaryIndentation();
-		void parseCurrentLine(const string& line);
-		void processProcessor(string& line);
-		void registerInStatementIndent(const string& line, int i, int spaceTabCount,
+		void parseCurrentLine(const string &line);
+		void popLastInStatementIndent();
+		void processPreProcessor(string &line);
+		void registerInStatementIndent(const string &line, int i, int spaceIndentCount,
 		                               int tabIncrementIn, int minIndent, bool updateParenStack);
 		void initVectors();
 		string preLineWS(int spaceTabCount_, int tabCount_);
+		int  adjustIndentCountForBreakElseIfComments() const;
 		int  convertTabToSpaces(int i, int tabIncrementIn) const;
-		int  getInStatementIndentAssign(const string& line, size_t currPos) const;
-		int  getInStatementIndentComma(const string& line, size_t currPos) const;
-		bool isClassAccessModifier(const string& line) const;
-		bool isIndentedPreprocessor(const string& line, size_t currPos) const;
-		bool isLineEndComment(const string& line, int startPos) const;
-		bool statementEndsWithComma(const string& line, int index) const;
-		vector<vector<const string*>*>* copyTempStacks(const ASBeautifier& other) const;
-		template<typename T> void deleteContainer(T& container);
+		int  getInStatementIndentAssign(const string &line, size_t currPos) const;
+		int  getInStatementIndentComma(const string &line, size_t currPos) const;
+		bool isClassAccessModifier(const string &line) const;
+		bool isIndentedPreprocessor(const string &line, size_t currPos) const;
+		bool isLineEndComment(const string &line, int startPos) const;
+		bool isPreprocessorDefinedCplusplus(const string &line) const;
+		bool isInPreprocessorUnterminatedComment(const string &line);
+		bool statementEndsWithComma(const string &line, int index) const;
+		vector<vector<const string*>*>* copyTempStacks(const ASBeautifier &other) const;
+		template<typename T> void deleteContainer(T &container);
 		void deleteContainer(vector<vector<const string*>*>* &container);
-		template<typename T> void initContainer(T& container, T value);
+		template<typename T> void initContainer(T &container, T value);
 
 	private:  // variables
-
 		int beautifierFileType;
 		vector<const string*>* headers;
 		vector<const string*>* nonParenHeaders;
@@ -401,19 +423,19 @@ class ASBeautifier : protected ASResource, protected ASBase
 		vector<const string*>* nonAssignmentOperators;
 		vector<const string*>* indentableHeaders;
 
-		vector<ASBeautifier*> *waitingBeautifierStack;
-		vector<ASBeautifier*> *activeBeautifierStack;
-		vector<int> *waitingBeautifierStackLengthStack;
-		vector<int> *activeBeautifierStackLengthStack;
-		vector<const string*> *headerStack;
-		vector< vector<const string*>* > *tempStacks;
-		vector<int> *blockParenDepthStack;
-		vector<bool> *blockStatementStack;
-		vector<bool> *parenStatementStack;
-		vector<bool> *bracketBlockStateStack;
-		vector<int> *inStatementIndentStack;
-		vector<int> *inStatementIndentStackSizeStack;
-		vector<int> *parenIndentStack;
+		vector<ASBeautifier*>* waitingBeautifierStack;
+		vector<ASBeautifier*>* activeBeautifierStack;
+		vector<int>* waitingBeautifierStackLengthStack;
+		vector<int>* activeBeautifierStackLengthStack;
+		vector<const string*>* headerStack;
+		vector< vector<const string*>* >* tempStacks;
+		vector<int>* blockParenDepthStack;
+		vector<bool>* blockStatementStack;
+		vector<bool>* parenStatementStack;
+		vector<bool>* bracketBlockStateStack;
+		vector<int>* inStatementIndentStack;
+		vector<int>* inStatementIndentStackSizeStack;
+		vector<int>* parenIndentStack;
 
 		ASSourceIterator* sourceIterator;
 		const string* currentHeader;
@@ -428,6 +450,7 @@ class ASBeautifier : protected ASResource, protected ASBase
 		bool isInAsmOneLine;
 		bool isInAsmBlock;
 		bool isInComment;
+		bool isInPreprocessorComment;
 		bool isInHorstmannComment;
 		bool isInCase;
 		bool isInQuestion;
@@ -453,7 +476,8 @@ class ASBeautifier : protected ASResource, protected ASBase
 		bool shouldForceTabIndentation;
 		bool emptyLineFill;
 		bool backslashEndsPrevLine;
-		bool lineOpensComment;
+		bool lineOpensWithLineComment;
+		bool lineOpensWithComment;
 		bool blockCommentNoIndent;
 		bool blockCommentNoBeautify;
 		bool previousLineProbationTab;
@@ -462,8 +486,8 @@ class ASBeautifier : protected ASResource, protected ASBase
 		bool isInClass;
 		bool isInSwitch;
 		bool foundPreCommandHeader;
-		int  tabCount;
-		int  spaceTabCount;
+		int  indentCount;
+		int  spaceIndentCount;
 		int  lineOpeningBlocksNum;
 		int  lineClosingBlocksNum;
 		int  fileType;
@@ -471,13 +495,15 @@ class ASBeautifier : protected ASResource, protected ASBase
 		int  minConditionalIndent;
 		int  parenDepth;
 		int  indentLength;
+		int  tabLength;
 		int  blockTabCount;
 		int  maxInStatementIndent;
-		int  classInitializerTabs;
+		int  classInitializerIndents;
 		int  templateDepth;
-		int  prevFinalLineSpaceTabCount;
-		int  prevFinalLineTabCount;
-		int  defineTabCount;
+		int  preprocessorCppExternCBracket;
+		int  prevFinalLineSpaceIndentCount;
+		int  prevFinalLineIndentCount;
+		int  defineIndentCount;
 		char quoteChar;
 		char prevNonSpaceCh;
 		char currentNonSpaceCh;
@@ -494,13 +520,15 @@ class ASEnhancer : protected ASBase
 	public:  // functions
 		ASEnhancer();
 		virtual ~ASEnhancer();
-		void init(int, int, string, bool, bool, bool);
-		void enhance(string& line, bool isInPreprocessor, bool isInSQL);
+		void init(int, int, int, bool, bool, bool, bool, bool);
+		void enhance(string &line, bool isInPreprocessor, bool isInSQL);
 
 	private:
 		// options from command line or options file
 		int  indentLength;
+		int  tabLength;
 		bool useTabs;
+		bool forceTab;
 		bool caseIndent;
 		bool preprocessorIndent;
 		bool emptyLineFill;
@@ -539,13 +567,15 @@ class ASEnhancer : protected ASBase
 
 
 	private:  // functions
-		size_t  findCaseColon(string&  line, size_t caseIndex) const;
-		int     indentLine(string&  line, int indent) const;
-		bool    isBeginDeclareSectionSQL(string&  line, size_t index) const;
-		bool    isEndDeclareSectionSQL(string&  line, size_t index) const;
-		bool    isOneLineBlockReached(string& line, int startChar) const;
-		size_t  processSwitchBlock(string&  line, size_t index);
-		int     unindentLine(string&  line, int unindent) const;
+		void    convertForceTabIndentToSpaces(string  &line) const;
+		void    convertSpaceIndentToForceTab(string &line) const;
+		size_t  findCaseColon(string  &line, size_t caseIndex) const;
+		int     indentLine(string  &line, int indent) const;
+		bool    isBeginDeclareSectionSQL(string  &line, size_t index) const;
+		bool    isEndDeclareSectionSQL(string  &line, size_t index) const;
+		bool    isOneLineBlockReached(string &line, int startChar) const;
+		size_t  processSwitchBlock(string  &line, size_t index);
+		int     unindentLine(string  &line, int unindent) const;
 };  // Class ASEnhancer
 
 //-----------------------------------------------------------------------------
@@ -566,16 +596,20 @@ class ASFormatter : public ASBeautifier
 		void setAddBracketsMode(bool state);
 		void setAddOneLineBracketsMode(bool state);
 		void setBracketFormatMode(BracketMode mode);
+		void setBreakAfterMode(bool state);
 		void setBreakClosingHeaderBracketsMode(bool state);
 		void setBreakBlocksMode(bool state);
 		void setBreakClosingHeaderBlocksMode(bool state);
 		void setBreakElseIfsMode(bool state);
 		void setBreakOneLineBlocksMode(bool state);
+		void setCloseTemplatesMode(bool state);
 		void setDeleteEmptyLinesMode(bool state);
 		void setIndentCol1CommentsMode(bool state);
 		void setLineEndFormat(LineEndFormat fmt);
+		void setMaxCodeLength(int max);
 		void setOperatorPaddingMode(bool mode);
 		void setParensOutsidePaddingMode(bool mode);
+		void setParensFirstPaddingMode(bool mode);
 		void setParensInsidePaddingMode(bool mode);
 		void setParensHeaderPaddingMode(bool mode);
 		void setParensUnPaddingMode(bool state);
@@ -589,10 +623,10 @@ class ASFormatter : public ASBeautifier
 		int  getFormatterFileType() const;
 
 	private:  // functions
-		ASFormatter(const ASFormatter& copy);       // copy constructor not to be imlpemented
-		ASFormatter& operator=(ASFormatter&);       // assignment operator not to be implemented
-		template<typename T> void deleteContainer(T& container);
-		template<typename T> void initContainer(T& container, T value);
+		ASFormatter(const ASFormatter &copy);       // copy constructor not to be imlpemented
+		ASFormatter &operator=(ASFormatter &);      // assignment operator not to be implemented
+		template<typename T> void deleteContainer(T &container);
+		template<typename T> void initContainer(T &container, T value);
 		char peekNextChar() const;
 		BracketType getBracketType();
 		bool addBracketsToStatement();
@@ -608,31 +642,37 @@ class ASFormatter : public ASBeautifier
 		bool isClosingHeader(const string* header) const;
 		bool isCurrentBracketBroken() const;
 		bool isDereferenceOrAddressOf() const;
-		bool isExecSQL(string&  line, size_t index) const;
-		bool isEmptyLine(const string& line) const;
+		bool isExecSQL(string  &line, size_t index) const;
+		bool isEmptyLine(const string &line) const;
 		bool isNextWordSharpNonParenHeader(int startChar) const;
 		bool isNonInStatementArrayBracket() const;
+		bool isOkToSplitFormattedLine();
 		bool isPointerOrReference() const;
 		bool isPointerOrReferenceCentered() const;
 		bool isSharpStyleWithParen(const string* header) const;
-		bool isStructAccessModified(string&  firstLine, size_t index) const;
+		bool isStructAccessModified(string  &firstLine, size_t index) const;
 		bool isUnaryOperator() const;
 		bool isImmediatelyPostCast() const;
 		bool isInExponent() const;
 		bool isNextCharOpeningBracket(int startChar) const;
 		bool isOkToBreakBlock(BracketType bracketType) const;
+		bool pointerSymbolFollows() const;
 		int  getCurrentLineCommentAdjustment();
 		int  getNextLineCommentAdjustment();
-		int  isOneLineBlockReached(string& line, int startChar) const;
+		int  isOneLineBlockReached(string &line, int startChar) const;
 		void adjustComments();
+		void appendChar(char ch, bool canBreakLine);
 		void appendCharInsideComments();
-		void appendSequence(const string& sequence, bool canBreakLine = true);
+		void appendOperator(const string &sequence, bool canBreakLine = true);
+		void appendSequence(const string &sequence, bool canBreakLine = true);
 		void appendSpacePad();
 		void appendSpaceAfter();
-		void breakLine();
+		void breakLine(bool isSplitLine=false);
 		void buildLanguageVectors();
-		void checkForHeaderFollowingComment(const string& firstLine);
+		void updateFormattedLineSplitPoints(char appendedChar);
+		void updateFormattedLineSplitPointsOperator(const string &sequence);
 		void checkIfTemplateOpener();
+		void clearFormattedLineSplitPoints();
 		void convertTabToSpaces();
 		void deleteContainer(vector<BracketType>* &container);
 		void formatArrayRunIn();
@@ -648,6 +688,9 @@ class ASFormatter : public ASBeautifier
 		void formatQuoteOpener();
 		void formatPointerOrReference();
 		void formatPointerOrReferenceCast();
+		void formatPointerOrReferenceToMiddle();
+		void formatPointerOrReferenceToName();
+		void formatPointerOrReferenceToType();
 		void fixOptionVariableConflicts();
 		void goForward(int i);
 		void isLineBreakBeforeClosingHeader();
@@ -658,11 +701,15 @@ class ASFormatter : public ASBeautifier
 		void processPreprocessor();
 		void setAttachClosingBracket(bool state);
 		void setBreakBlocksVariables();
+		void testForTimeToSplitFormattedLine();
 		void trimContinuationLine();
-		size_t findNextChar(string& line, char searchChar, int searchStart = 0);
+		void updateFormattedLineSplitPointsPointerOrReference(size_t index);
+		size_t findFormattedLineSplitPoint() const;
+		size_t findNextChar(string &line, char searchChar, int searchStart = 0);
+		const string* checkForHeaderFollowingComment(const string &firstLine) const;
 		const string* getFollowingOperator() const;
-		string getPreviousWord(const string& line, int currPos) const;
-		string peekNextText(const string& firstLine, bool endOnEmptyLine=false, bool shouldReset=false) const;
+		string getPreviousWord(const string &line, int currPos) const;
+		string peekNextText(const string &firstLine, bool endOnEmptyLine=false, bool shouldReset=false) const;
 
 	private:  // variables
 		int formatterFileType;
@@ -677,10 +724,10 @@ class ASFormatter : public ASBeautifier
 		ASSourceIterator* sourceIterator;
 		ASEnhancer* enhancer;
 
-		vector<const string*> *preBracketHeaderStack;
-		vector<BracketType> *bracketTypeStack;
-		vector<int> *parenStack;
-		vector<bool> *structStack;
+		vector<const string*>* preBracketHeaderStack;
+		vector<BracketType>* bracketTypeStack;
+		vector<int>* parenStack;
+		vector<bool>* structStack;
 		string readyFormattedLine;
 		string currentLine;
 		string formattedLine;
@@ -701,9 +748,23 @@ class ASFormatter : public ASBeautifier
 		int  traceLineNumber;
 		size_t checksumIn;
 		size_t checksumOut;
-		size_t leadingSpaces;
-		size_t formattedLineCommentNum;     // comment location on formattedLine
 		size_t currentLineFirstBracketNum;	// first bracket location on currentLine
+		size_t formattedLineCommentNum;     // comment location on formattedLine
+		size_t leadingSpaces;
+		size_t maxCodeLength;
+
+		// possible split points
+		size_t maxSemi;			// probably a 'for' statement
+		size_t maxAndOr;		// probably an 'if' statement
+		size_t maxComma;
+		size_t maxParen;
+		size_t maxWhiteSpace;
+		size_t maxSemiPending;
+		size_t maxAndOrPending;
+		size_t maxCommaPending;
+		size_t maxParenPending;
+		size_t maxWhiteSpacePending;
+
 		size_t previousReadyFormattedLineLength;
 		FormatStyle formattingStyle;
 		BracketMode bracketFormatMode;
@@ -711,18 +772,21 @@ class ASFormatter : public ASBeautifier
 		PointerAlign pointerAlignment;
 		ReferenceAlign referenceAlignment;
 		LineEndFormat lineEnd;
-		bool computeChecksumIn(const string& currentLine_);
-		bool computeChecksumOut(const string& beautifiedLine);
+		bool computeChecksumIn(const string &currentLine_);
+		bool computeChecksumOut(const string &beautifiedLine);
 		bool isVirgin;
 		bool shouldPadOperators;
 		bool shouldPadParensOutside;
+		bool shouldPadFirstParen;
 		bool shouldPadParensInside;
 		bool shouldPadHeader;
 		bool shouldUnPadParens;
 		bool shouldConvertTabs;
 		bool shouldIndentCol1Comments;
+		bool shouldCloseTemplates;
 		bool isInLineComment;
 		bool isInComment;
+		bool isInCommentStartLine;
 		bool noTrimCommentContinuation;
 		bool isInPreprocessor;
 		bool isInPreprocessorBeautify;
@@ -737,6 +801,8 @@ class ASFormatter : public ASBeautifier
 		bool isInVerbatimQuote;
 		bool haveLineContinuationChar;
 		bool isInQuoteContinuation;
+		bool isInBlParen;
+		bool isHeaderInMultiStatementLine;
 		bool isSpecialChar;
 		bool isNonParenHeader;
 		bool foundQuestionMark;
@@ -748,13 +814,16 @@ class ASFormatter : public ASBeautifier
 		bool foundPreCommandHeader;
 		bool foundCastOperator;
 		bool isInLineBreak;
+		bool endOfAsmReached;
 		bool endOfCodeReached;
 		bool lineCommentNoIndent;
+		bool isInEnum;
 		bool isInExecSQL;
 		bool isInAsm;
 		bool isInAsmOneLine;
 		bool isInAsmBlock;
 		bool isLineReady;
+		bool elseHeaderFollowsComments;
 		bool isPreviousBracketBlockRelated;
 		bool isInPotentialCalculation;
 		bool isCharImmediatelyPostComment;
@@ -764,6 +833,7 @@ class ASFormatter : public ASBeautifier
 		bool isCharImmediatelyPostCloseBlock;
 		bool isCharImmediatelyPostTemplate;
 		bool isCharImmediatelyPostReturn;
+		bool isCharImmediatelyPostThrow;
 		bool isCharImmediatelyPostOperator;
 		bool isCharImmediatelyPostPointerOrReference;
 		bool breakCurrentOneLineBlock;
@@ -775,11 +845,13 @@ class ASFormatter : public ASBeautifier
 		bool shouldBreakOneLineStatements;
 		bool shouldBreakClosingHeaderBrackets;
 		bool shouldBreakElseIfs;
+		bool shouldBreakLineAfterLogical;
 		bool shouldAddBrackets;
 		bool shouldAddOneLineBrackets;
 		bool shouldDeleteEmptyLines;
 		bool needHeaderOpeningBracket;
 		bool shouldBreakLineAtNextChar;
+		bool shouldKeepLineUnbroken;
 		bool passedSemicolon;
 		bool passedColon;
 		bool isImmediatelyPostNonInStmt;
@@ -789,33 +861,23 @@ class ASFormatter : public ASBeautifier
 		bool isImmediatelyPostEmptyBlock;
 		bool isImmediatelyPostPreprocessor;
 		bool isImmediatelyPostReturn;
+		bool isImmediatelyPostThrow;
 		bool isImmediatelyPostOperator;
 		bool isImmediatelyPostTemplate;
 		bool isImmediatelyPostPointerOrReference;
-
 		bool shouldBreakBlocks;
 		bool shouldBreakClosingHeaderBlocks;
 		bool isPrependPostBlockEmptyLineRequested;
 		bool isAppendPostBlockEmptyLineRequested;
-
 		bool prependEmptyLine;
 		bool appendOpeningBracket;
 		bool foundClosingHeader;
-
 		bool isInHeader;
 		bool isImmediatelyPostHeader;
 		bool isInCase;
 		bool isJavaStaticConstructor;
 
 	private:  // inline functions
-		// append a character to the current formatted line.
-		void appendChar(char ch, bool canBreakLine) {
-			if (canBreakLine && isInLineBreak)
-				breakLine();
-			formattedLine.append(1, ch);
-			isImmediatelyPostCommentOnly = false;
-		}
-
 		// append the CURRENT character (curentChar) to the current formatted line.
 		void appendCurrentChar(bool canBreakLine = true) {
 			appendChar(currentChar, canBreakLine);
